var ref = require('ssb-ref')
var Repo = require('./lib/repo')
var pull = require('pull-stream')
var LRUCache = require('lrucache')

exports.Repo = Repo

// Note: these caches are shared by all ssb-git-repo instances, for
// performance purposes. If this becomes a problem, we could namespace the keys
// by the repo id.
var globalCaches = {
  pack: new LRUCache(5), // packfile contents
  idx: new LRUCache(100), // pack index file contents
  offset: new LRUCache(500), // locations of objects in packfiles
  object: new LRUCache(25), // objects contents
}

exports.createRepo = function (sbot, options, cb) {
  if (typeof options == 'function') cb = options, options = null
  if (!options) options = {}
  var msg = {
    type: 'git-repo'
  }
  if (options.name)
    msg.name = options.name
  if (!options.caches)
    options.caches = globalCaches
  if (!options.upstream)
    gotUpstream(null, null)
  else if (options.upstream.id)
    gotUpstream(null, options.upstream)
  else {
    if (!ref.isMsg(options.upstream))
      return cb(new Error('Invalid repo ID: ' + options.upstream))
    exports.getRepo(sbot, options.upstream, options, gotUpstream)
  }
  function gotUpstream(err, upstream) {
    if (err) return cb(err)
    if (upstream) msg.upstream = upstream.id
    sbot.publish(msg, function (err, msg) {
      if (err) return cb(err)
      var repo = new Repo(sbot, msg.key, msg.value, upstream, options)
      repo._sync(options.live)
      cb(null, repo)
    })
  }
}

exports.getRepo = function (sbot, id, options, cb) {
  if (typeof options == 'function') cb = options, options = null
  if (!options) options = {}
  if (!options.caches) options.caches = globalCaches
  var msg

  if (typeof id == 'object') {
    var msg = id.value
    id = id.key
    gotMsg(null, msg)
  } else {
    sbot.get(id, gotMsg)
  }

  function gotMsg(err, _msg) {
    if (err) return cb(err)
    msg = _msg
    if (msg.content.upstream)
      exports.getRepo(sbot, msg.content.upstream, options, gotUpstream)
    else
      gotUpstream()
  }

  function gotUpstream(err, upstream) {
    if (err) return cb(err)
    var repo = new Repo(sbot, id, msg, upstream, options)
    repo._sync(options.live)
    if (upstream) upstream._sync(options.live)
    cb(null, repo)
  }
}

exports.repos = function (sbot, options) {
  if (!options.caches) options.caches = globalCaches
  return pull(
    sbot.createFeedStream(options),
    pull.filter(function (msg) {
      return msg.value.content.type === 'git-repo'
    }),
    pull.map(function (msg) {
      return new Repo(sbot, msg.key, msg.value, null, options)
    })
  )
}
